#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "stdint.h"
#include "main.h"
#include "driver/adc.h"
#include "driver/rs232.h"
#include "lowlevel.h"

void LED(uint8_t num, uint8_t state) {
    switch (num) {
        case 1: switch (state) {
            case 0: LED1_PORT &= ~_BV(LED1); break;
            case 1: LED1_PORT |= _BV(LED1); break;
            case 2: LED1_PORT ^= _BV(LED1); break;
            } break;

        case 2: switch (state) {
            case 0: LED2_PORT &= ~_BV(LED2); break;
            case 1: LED2_PORT |= _BV(LED2); break;
            case 2: LED2_PORT ^= _BV(LED2); break;
            } break;

        case 3: switch (state) {
            case 0: LED3_PORT &= ~_BV(LED3); break;
            case 1: LED3_PORT |= _BV(LED3); break;
            case 2: LED3_PORT ^= _BV(LED3); break;
            } break;

        case 4: switch (state) {
            case 0: LED4_PORT &= ~_BV(LED4); break;
            case 1: LED4_PORT |= _BV(LED4); break;
            case 2: LED4_PORT ^= _BV(LED4); break;
            } break;

        case 5: switch (state) {
            case 0: LED5_PORT &= ~_BV(LED5); break;
            case 1: LED5_PORT |= _BV(LED5); break;
            case 2: LED5_PORT ^= _BV(LED5); break;
            } break;
    }
}

void init_hardware(void) {
    RS232_init(); // initialize RS485 interface
    RS232_puts_p(PSTR("CARRERA beta loading\n"));

    initADC();

    // set LED output
    DDR(LED1_PORT) |= _BV(LED1);
    DDR(LED2_PORT) |= _BV(LED2);
    DDR(LED3_PORT) |= _BV(LED3);
    DDR(LED4_PORT) |= _BV(LED4);
    DDR(LED5_PORT) |= _BV(LED5);

    // set Controller Input Pull-UPs
    CONTROLLER_PORT |= (_BV(CONTROLLER1_SW) | _BV(CONTROLLER2_SW) | _BV(CONTROLLER3_SW) | _BV(CONTROLLER4_SW));

    // switch pull-ups
    SW_FUEL_PORT |= _BV(SW_FUEL);

    //RAIL_DETECT_PORT |= _BV(RAIL_DETECT); // enable internal pull-up
    DDR(RAIL_POWER_PORT) |= _BV(RAIL_POWER);


    LED(1, 1); _delay_ms(50);
    LED(2, 1); _delay_ms(50);
    LED(3, 1); _delay_ms(50);
    LED(4, 1); _delay_ms(50);
    LED(5, 1); _delay_ms(50);
    LED(1, 0); _delay_ms(50);
    LED(2, 0); _delay_ms(50);
    LED(3, 0); _delay_ms(50);
    LED(4, 0); _delay_ms(50);
    LED(5, 0); _delay_ms(50);

    //TCCR0 = (1<<CS01); //divide by 8


    // setup data bit timer
    TCCR2 = (1<<CS21); //divide by 8
    TCCR2 |= (1<<WGM21); // set compare match
    OCR2 = TIMER2_50US;
    TIMSK |= 1<<OCIE2; //enable timer2 interrupt

    // setup data packet timer
    //TCCR1A = (1<<COM1A1);
    TCCR1B = (1<<CS11); //divide by 8
    TCCR1B |= (1<<WGM12); // set compare match
    //TCCR1B = (1<<CS11) | (1<<CS10); //divide by 64
    //TCNT1 = TIMER_7500NS;
    OCR1A = TIMER1_7500NS;
    TIMSK |= 1<<OCIE1A; //enable timer1 interrupt

    RS232_puts_p(PSTR("INIT OK\n"));

}



void check_rails_shortcut(void) {
    // check for short circuit on the rails
    if ((PIN(RAIL_DETECT_PORT) & _BV(RAIL_DETECT)) == 0) {
        _delay_ms(2);
        if ((PIN(RAIL_DETECT_PORT) & _BV(RAIL_DETECT)) == 0) {
            cli(); // disable ALL Interrupts
            RAIL_POWER_PORT &= ~_BV(RAIL_POWER); // disable rails power
            RS232_puts_p(PSTR("!!! SHORT CIRCUIT ON RAILS, POWERED OFF !!!\n"));
            LED(1, 1);
            LED(2, 1);
            LED(3, 0);
            LED(4, 0);
            LED(5, 0);
            while (1) {
                LED(1, 2);
                LED(2, 2);
                LED(4, 2);
                LED(5, 2);
                _delay_ms(100);
                LED(3, 2);
                _delay_ms(100);
                LED(3, 2);
                _delay_ms(100);
                LED(3, 2);
                _delay_ms(100);
                LED(3, 2);
            }
        }
    }
}